@@ -4,19 +4,22 @@ Building Block
 
 """
 
+from __future__ import annotations
 
 import logging
 import os
-import rdkit.Chem.AllChem as rdkit
+from collections.abc import Collection
 from functools import partial
-import numpy as np
+from typing import Iterable, Iterator, Optional, Union
 
-from ..functional_groups import FunctionalGroup
+import numpy as np
+import rdkit.Chem.AllChem as rdkit
+
+from ...utilities import flatten, remake
 from ..atoms import Atom
 from ..bonds import Bond
+from ..functional_groups import FunctionalGroup
 from .molecule import Molecule
-from ...utilities import remake, flatten
-
 
 logger = logging.getLogger(__name__)
 
@@ -391,6 +394,11 @@ class BuildingBlock(Molecule):
                 functional_groups=building_block._functional_groups,
             )
         )
+        building_block._core_ids = frozenset(
+            building_block._get_core_ids(
+                functional_groups=building_block._functional_groups,
+            )
+        )
         return building_block
 
     @classmethod
@@ -607,24 +615,29 @@ class BuildingBlock(Molecule):
             placer_ids=placer_ids,
             functional_groups=self._functional_groups,
         )
+        self._core_ids = frozenset(self._get_core_ids(
+            functional_groups=self._functional_groups,
+        ))
 
-    def _normalize_placer_ids(self, placer_ids, functional_groups):
+    def _normalize_placer_ids(
+        self,
+        placer_ids: Optional[tuple[int, ...]],
+        functional_groups: Collection[FunctionalGroup],
+    ) -> frozenset[int]:
         """
         Get the final *placer* ids.
 
-        Parameters
-        ----------
-        placer_ids : :class:`tuple` of :class:`int`
-            The ids of *placer* atoms or ``None``.
+        Parameters:
 
-        functional_groups : :class:`iterable`
-            The :class:`.FunctionalGroup` instances of the building
-            block.
+            placer_ids: The ids of *placer* atoms or ``None``.
 
-        Returns
-        -------
-        :class:`tuple` of :class:`int`
-            Depending on the input  values, this function will return
+            functional_groups:
+                The :class:`.FunctionalGroup` instances of the building
+                block.
+
+        Returns:
+
+            Depending on the input values, this function will return
             different things.
 
             #. `placer_ids` is a :class:`tuple` of :class`int`: the
@@ -641,15 +654,50 @@ class BuildingBlock(Molecule):
         """
 
         if placer_ids is not None:
-            return placer_ids
+            return frozenset(placer_ids)
 
         if functional_groups:
-            return tuple(flatten(
+            return frozenset(flatten(
                 functional_group.get_placer_ids()
                 for functional_group in functional_groups
             ))
 
-        return tuple(atom.get_id() for atom in self._atoms)
+        return frozenset(atom.get_id() for atom in self._atoms)
+
+    def _get_core_ids(
+        self,
+        functional_groups: Iterable[FunctionalGroup],
+    ) -> Iterator[int]:
+        """
+        Get the final *core* ids.
+
+        This method may return duplicate ids.
+
+        Parameters:
+
+            functional_groups:
+                The :class:`.FunctionalGroup` instances of the building
+                block.
+
+        Yields:
+
+            The id of an atom defining the core of the molecule.
+
+        """
+
+        functional_group_atom_ids = {
+            atom_id
+            for functional_group in functional_groups
+            for atom_id in functional_group.get_atom_ids()
+        }
+        for atom in self._atoms:
+            atom_id = atom.get_id()
+            if atom_id not in functional_group_atom_ids:
+                yield atom_id
+
+        for functional_group in functional_groups:
+            for atom_id in functional_group.get_core_atom_ids():
+                yield atom_id
 
     def _extract_functional_groups(self, functional_groups):
         """
@@ -689,10 +737,7 @@ class BuildingBlock(Molecule):
 
         """
 
-        atom_map = {a.get_id(): a for a in self._atoms}
-        self._functional_groups = tuple(
-            fg.with_atoms(atom_map) for fg in functional_groups
-        )
+        self._functional_groups = tuple(functional_groups)
         return self
 
     def with_functional_groups(self, functional_groups):
@@ -717,18 +762,22 @@ class BuildingBlock(Molecule):
     def _with_canonical_atom_ordering(self):
         ordering = rdkit.CanonicalRankAtoms(self.to_rdkit_mol())
         super()._with_canonical_atom_ordering()
-        atom_map = {
-            old_id: self._atoms[new_id]
+        id_map = {
+            old_id: new_id
             for old_id, new_id in enumerate(ordering)
         }
         self._functional_groups = tuple(
-            functional_group.with_atoms(atom_map)
+            functional_group.with_ids(id_map)
             for functional_group in self._functional_groups
         )
-        self._placer_ids = tuple(
-            atom_map[placer_id].get_id()
+        self._placer_ids = frozenset(
+            id_map[placer_id]
             for placer_id in self._placer_ids
         )
+        self._core_ids = frozenset(
+            id_map[core_id]
+            for core_id in self._core_ids
+        )
         return self
 
     def get_num_functional_groups(self):
@@ -744,21 +793,23 @@ class BuildingBlock(Molecule):
 
         return len(self._functional_groups)
 
-    def get_functional_groups(self, fg_ids=None):
+    def get_functional_groups(
+        self,
+        fg_ids: Optional[Union[int, Iterable[int]]] = None,
+    ) -> Iterable[FunctionalGroup]:
         """
         Yield the functional groups, ordered by id.
 
-        Parameters
-        ----------
-        fg_ids : :class:`iterable` of :class:`int`, optional
-            The ids of functional groups yielded. If ``None``, then
-            all functional groups are yielded. Can be a single
-            :class:`int`, if a single functional group is
-            desired.
+        Parameters:
+
+            fg_ids:
+                The ids of functional groups yielded. If ``None``, then
+                all functional groups are yielded. Can be a single
+                :class:`int`, if a single functional group is
+                desired.
+
+        Yields:
 
-        Yields
-        ------
-        :class:`.FunctionalGroup`
             A functional group of the building block.
 
         """
@@ -775,8 +826,21 @@ class BuildingBlock(Molecule):
         clone = super().clone()
         clone._functional_groups = self._functional_groups
         clone._placer_ids = self._placer_ids
+        clone._core_ids = self._core_ids
         return clone
 
+    def get_num_placers(self) -> int:
+        """
+        Return the number of *placer* atoms in the building block.
+
+        Returns:
+
+            The number of *placer* atoms in the building block.
+
+        """
+
+        return len(self._placer_ids)
+
     def get_placer_ids(self):
         """
         Yield the ids of *placer* atoms.
@@ -808,7 +872,7 @@ class BuildingBlock(Molecule):
         Yields
         ------
         :class:`int`
-            The id of a core atom.
+            The id of a *core* atom.
 
         See Also
         --------
@@ -816,19 +880,7 @@ class BuildingBlock(Molecule):
 
         """
 
-        functional_group_atom_ids = {
-            atom_id
-            for functional_group in self._functional_groups
-            for atom_id in functional_group.get_atom_ids()
-        }
-        for atom in self._atoms:
-            atom_id = atom.get_id()
-            if atom_id not in functional_group_atom_ids:
-                yield atom_id
-
-        for functional_group in self._functional_groups:
-            for atom_id in functional_group.get_core_atom_ids():
-                yield atom_id
+        yield from self._core_ids
 
     def __str__(self):
         if self._functional_groups:
