@@ -4,7 +4,7 @@ JSON Deserialization Utilities
 
 """
 
-from stk.molecular import Atom, Bond, AtomInfo, BondInfo
+from stk.molecular import Atom, AtomInfo, Bond, BondInfo
 
 
 def to_atom(id, json):
